#
# inetclient.rb version 1.3.3
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

begin
  require 'net/protocol'
rescue LoadError
  require 'net/session'
end

require 'amstd/extmod'
require 'amstd/must'
require 'amstd/bug'
require 'amstd/dispatch'


class INETclient

  Version = '1.3.3'

  def login( *args )
    return if logged_in?
    if NoPass then
      @session.start( *args )
    else
      @session.start( INETsocket, *args )
    end
    do_login
  end

  def do_login()  end; private :do_login
  def do_logout() end; private :do_logout

  def logout
    return unless logged_in?
    do_logout
    @session.finish
  end

  def logged_in?
    @session.active?
  end

  class << self
    def login( addr, port, *args )
      cl = new( addr, port )
      cl.login( *args )
      if iterator?
        yield cl
        cl.logout
      end
    end
  end

  def address
    @session.address
  end

  def port
    @session.port
  end

  def socket
    @session.socket
  end

  def proto_type
    @session.proto_type
  end

  def proto
    @session.proto
  end

  def proto=( arg )
    @session.proto = arg
  end

end

INETClient = INETclient


if defined? Net::Version then
  v = Net::Version
else
  v = Net::Protocol::Version
end
major, minor, teeny = v.split('.').collect {|i| i.to_i }

if major < 1 then
  INETclient.const_set :NoPass, false
else
  INETclient.const_set :NoPass, true
end

if major <= 1 then
  if minor <= 1 then
    if minor < 1 or teeny < 5 then         # v < 1.1.5
      c = Net::ProtocolSocket
    elsif teeny >= 5 and teeny < 27 then   # 1.1.5 <= v < 1.1.27
      c = Net::Socket
    end
  end
end
unless c then                              # v >= 1.1.27
  c = Net::NetPrivate::Socket
end

class INETsocket < c

  include Dispatchable

  def initialize( addr, port, pipe = nil )
    addr.must String
    port.must Integer
    super( addr, port, pipe )

    @pipe << "TCP connection to #{addr} opened" if @pipe
  end

  property :socket, TCPsocket, true, true

  event_handler :on_writing
  event_handler :on_write
  event_handler :on_writeall

  event_handler :on_reading
  event_handler :on_read
  event_handler :on_readall


  def writeline( str )
    str.must String
    super
  end


  private

  def do_write_beg
    super
    dispatch :on_writing
  end

  def do_write_do( arg )
    ret = super
    dispatch :on_write, ret
    return ret
  end

  def do_write_fin
    ret = super
    dispatch :on_writeall, ret
    return ret
  end


  public

  def read( len, &blk )
    do_read(blk){ super( len ) }
  end

  def readuntil( targ, &blk )
    do_read(blk){ super( targ ) }
  end
      
  def read_pendstr( ret = '', &blk )
    do_read(blk){ super( ret ) }
  end

  def read_pendlist
    do_read(nil){ super }
  end


  private

  def do_read( block )
    @rlen = 0
    push_event( :on_read, block ) if block
    dispatch :on_reading
    ret = yield
    dispatch :on_readall
    pop_event( :on_read ) if block

    return ret
  end

  def fill_rbuf
    tmp = @socket.sysread( READ_BLOCK )
    @buffer << tmp
                                  #$stderr.puts ">>>sysread size=#{tmp.size}"
    dispatch :on_read, tmp.size
  end

end
