#
# popclient.rb version 1.3.3
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2 or later.
#

require 'inetclient'
require 'net/pop'


if INETclient::NoPass then
  class INET_POP3_C < Net::POP3
    protocol_param :socket_type, INETsocket
  end
  class INET_APOP_C < Net::APOP
    protocol_param :socket_type, INETsocket
  end
else
  INET_POP3_C = Net::POP3
  INET_APOP_C = Net::APOP
end


class POP3client < INETclient

  private

  def initialize( addr = 'localhost', port = 110 )
    @session = INET_POP3_C.new( addr, port )
  end

  def do_login
    @mails = []
    @session.each do |m|
      @mails.push POPmail.new(m)
    end
    @mails.freeze
  end

  def do_logout
    @mails.each do |item| item.do_proc end
    @mails = nil
  end


  public

  attr :mails

  def each
    @mails.each{|m| yield m }
  end


  class POPmail

    def initialize( m )
      @item = m
      @dele    = false
      @mail    = nil
      @header  = nil
    end

    def size
      @item.size
    end

    def mail( ret = '' )
      @mail = @item.pop( ret ) unless @mail
      @mail
    end
    alias pop mail
    alias all mail

    def header( ret = '' )
      @header = @item.top( 0, ret ) unless @header
      @header
    end

    def top( lines, ret = '' )
      @item.top( lines, ret )
    end

    def delete
      @dele = true
    end

    def delete!
      @item.delete! unless @deleted
      @dele = false
    end
        
    def undel
      if @item.deleted? then
        raise Net::ProtoCommandError, "can't undel deleted mail"
      end
      undel!
    end

    def undel!
      @dele = false
    end

    def delete?
      @dele
    end

    def deleted?
      @item.deleted?
    end

    def do_proc
      @item.delete! if @dele and not @item.deleted?
    end

  end

end

POP3Client = POP3client


class APOPclient < POP3client

  def initialize( addr = 'localhost', port = 110 )
    @session = INET_APOP_C.new( addr, port )
  end

end

APOPClient = APOPclient
